module Dockerploy
  class Deploy
    SSH_PORT = 22
    HTTP_PORT = 80

    def initialize(config, options = {})
      @config = config
      @options = options
    end

    def destroy(ssh_client, server)
      container_name = sprintf('%s_%s', @config.application_name, server[:container][:http_port])
      ssh_client.command(sprintf('docker rm -f %s', container_name))
    end

    def run(ssh_client, server)
      option_delimiter = ' '
      command = sprintf('docker run -d --name %s_%s', @config.application_name, server[:container][:http_port])
      command << option_delimiter + hostname_option
      command << option_delimiter + port_option(server)
      command << option_delimiter + volume_option
      command << option_delimiter + environment_variables_option
      custom_variables = custom_environment_variables_option(server)
      command << option_delimiter + custom_variables if custom_variables.length > 0
      command << option_delimiter + image_name
      ssh_client.command(command)
    end

    def deploy
      create_tag if @config.tagging?
      @config.servers.each do |server|
        run_before_hooks(server)
        ssh_client = SSHClient.new(server[:host], server[:username], server[:password], server[:port])
        destroy(ssh_client, server)
        run(ssh_client, server)
        run_after_hooks(server)
      end
    end

    def run_before_hooks(server)
      @config.before_all_hooks.each { |hook| run_hook(server, hook) }
      @config.before_hooks.each { |hook| run_hook(server, hook) }
    end

    def run_after_hooks(server)
      @config.after_all_hooks.each { |hook| run_hook(server, hook) }
      @config.after_hooks.each { |hook| run_hook(server, hook) }
    end

    def run_hook(server, hook)
      if hook[:local]
        hook[:local].each do |command|
          ShellClient.new(abort_on_failure: true).command(command)
        end
      elsif hook[:remote]
        hook[:remote].each do |command|
          SSHClient.new(server[:host], server[:username], server[:password], server[:port]).command(command)
        end
      end
    end
    private

    def create_tag
      tag_name = sprintf('%s-%s', @config.env, Time.now.strftime('%Y%m%dT%H%M'))
      ShellClient.new.command(sprintf('git tag %s %s', tag_name, @config.branch))
      ShellClient.new.command(sprintf('git push origin %s', tag_name))
    end

    def volume_option
      volumes = @config.volumes
      result = ''
      if volumes
        volumes.each { |volume| result << sprintf('-v %s:%s ', volume[:volume][:host], volume[:volume][:guest]) }
      end
      result
    end

    def environment_variables_option
      env_file = @config.env_file
      result = ''
      if env_file
        if File.exist?(env_file)
          environment_variables = YAML.load_file(env_file).symbolize_keys
          environment_variables.each do |k, v|
            result << sprintf("-e %s='%s' ", k.upcase, v)
          end
        end
      end
      result.strip
    end

    def custom_environment_variables_option(server)
      result = ''
      variables = server[:custom_variables]
      if variables
        variables.each_pair do |key, value|
          result << sprintf("-e %s='%s' ", key.upcase, value)
        end
      end
      result.strip
    end

    def port_option(server)
      container_config = server[:container]
      result = ''
      result << sprintf('-p %s:%s:%s ', container_config[:host], container_config[:ssh_port], SSH_PORT)
      result << sprintf('-p %s:%s:%s', container_config[:host], container_config[:http_port], HTTP_PORT)
    end

    def hostname_option
      sprintf('--hostname %s', @config.application_name)
    end

    def image_name
      tag = @options[:tag] || @config.image_tag
      tag ? sprintf('%s:%s', @config.image_name, tag) : @config.image_name
    end
  end
end